"""
Name: XCS_Main.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA 
Note: Code primarily based on python XCS authored by Jose Antonio Martin H. <jamartin@dia.fi.upm.es> tranlated from a version by Martin V. Butz (in Java 1.0) 
Contact: ryan.j.urbanowicz@darmouth.edu
Created: 3/20/2009
Updated: 10/15/2013
Description: Command line Main method for running the XCS algorithm.  Handles inputs, builds the environment, and runs the XCS algorithm
---------------------------------------------------------------------------------------------------------------------------------------------------------
XCS Version 1.0: A Python implementation of the XCS learning classifier system algorithm implemented for SNP genetic data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python

#Import modules
from XCS import *
from XCS_GHEnvironment import *
import sys
import time
argv = sys.argv

helpstr = """
Parameters:
    1:  Specify the environment - 'gh'
    2:  Training Dataset - '/SomePath/SomeTrainfile.txt'
    3:  Testing Dataset - '/SomePath/SomeTestfile.txt'
    4:  Output Filename (Learning Progress Tracking) - '/SomePath/SomeProgressName'
    5:  Output Filename (Final Pop Evaluation) - '/SomePath/SomeFinalPopName'
    6:  Coding Length - '1', '2', or '3'
    7:  Reward/Payoff - '1000'
    8:  Dataset Partitions for CV - '10'
    9:  Performance Tracking Cycles - '50'
    10: Learning iterations - '100000.500000.1000000'
    11: Maximum Population Size - '1000'
    12: Subsumption Setting - '0' or '1'
    13: Selection Method - '0' (roulete wheel) or '1' (tournament) 
    
Command Line Run Example:
psuedo code: python ./XCS_Main.py environment TrainData TestData OutProgName outPopName CodeBits Reward CVPartitions TrackingCycles LearningIterations MaxPop Subsumption Selection
actual code: python ./XCS_Main.py gh /SomePath/SomeTrainfile.txt /SomePath/SomeTestfile.txt /SomePath/SomeProgressName /SomePath/SomeFinalPopName 2 1000 10 50 1000000 1000 0 0

"""

graphPerformance = False # Built in graphing ability, currently not functional, but mechanism is in place. NOT MEANT TO BE USED ON CLUSTER.
numArgs = len(argv)
print "Arguments: " + str(numArgs)
if numArgs == 14:
    if argv[1]=='gh':
        print ("Format Training data: "+argv[2]+"  using a "+argv[6]+" bit coding scheme.")
        
        #Sets up up algorithm to be run.
        e = Environment()
        e = GHEnvironment(str(argv[2]), str(argv[3]), int(argv[6]), int(argv[7]))
        sampleSize = e.getNrSamples()
        xcs = XCS(e, argv[4], argv[5], int(argv[6]), int(argv[8]), graphPerformance)   

        #Figure out the iteration stops for evaluation, and the max iterations.
        iterList = argv[10].split('.')
        for i in range(len(iterList)):
            iterList[i] = int(iterList[i])
        lastIter = iterList[len(iterList)-1]

        #Set some XCS parameters.
        if argv[9] == 'Default':
            xcs.setTrackingIterations(sampleSize)
        else:
            xcs.setTrackingIterations(int(argv[9]))
        xcs.setNumberOfTrials(lastIter, iterList)
        xcs.setPopulationSize(int(argv[11]))
        xcs.setSubsumption(int(argv[12]))
        xcs.setSelection(int(argv[13]))  
        
        #Run the XCS Algorithm   
        xcs.runXCS()        
        
    else:
        print "There is no environment handeling code for the specified environment"
else:
    print (helpstr)
    sys.exit()

"""
    Additions/Modifications made to the original python XCS script
    1.) Added an environment module designed to handle data mining environments (SNP data input specific codings)
    2.) Added an option to use tournament selection for the GA
    3.) Added the ability to manipulate from the command line the following parameters - (number of iterations, epoch size, performance report size, the use of subsumption
        max population size, number of experiments (arbitrary to new script setup).
    4.) Added the ability to learn in epochs (runs of entire datasets) - more of an alternative learning and tracking perspective
    5.) Added the ability to perform cross validation partitioning and assessment of the dataset within this script. (perhaps this could later be better parallelized outside of this script in a cluster script.
    6.) Prints out a second output file that shows the current ruleset, along with summary info on the population performance.
"""
