"""
Name: XCS_Test.py
Author: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA 
Note: Code primarily based on python XCS authored by Jose Antonio Martin H. <jamartin@dia.fi.upm.es> tranlated from a version by Martin V. Butz (in Java 1.0) 
Contact: ryan.j.urbanowicz@darmouth.edu
Created: 3/20/2009
Updated: 10/15/2013
Description: A testing replacement for XCS_Main.py to run XCS from Eclipse.
---------------------------------------------------------------------------------------------------------------------------------------------------------
XCS Version 1.0: A Python implementation of the XCS learning classifier system algorithm implemented for SNP genetic data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python

#Import modules
from XCS import *
from XCS_GHEnvironment import *
    
def main():
    #RUN PARAMETERS
    graphPerformance = True
    trainData = "2_1000_0_1600_0_0_CV_0_Train.txt"
    testData = "2_1000_0_1600_0_0_CV_0_Test.txt"
    outProg = "GH_XCS_ProgressTrack"
    outPop = "GH_XCS_PopulationOut"
    bitLength = 1    
    reward = 1000
    CVpartitions = 10
    iterInput = '8000'
    #trackCycles = 'Default' # set tracking cycles to number of data samples - trackCycles = 100 
    trackCycles = 100
    pop = 3200
    sub = 0
    select = 0
    
    #Figure out the iteration stops for evaluation, and the max iterations.
    iterList = iterInput.split('.')
    for i in range(len(iterList)):
        iterList[i] = int(iterList[i])
    lastIter = iterList[len(iterList)-1]
    
    #Sets up up algorithm to be run.
    e = GHEnvironment(trainData,testData,bitLength,reward)
    sampleSize = e.getNrSamples()
    xcs = XCS(e, outProg, outPop, bitLength, CVpartitions, graphPerformance)
    
    #Set some XCS parameters.
    if trackCycles == 'Default':
        xcs.setTrackingIterations(sampleSize)
    else:
        xcs.setTrackingIterations(trackCycles)
    xcs.setNumberOfTrials(lastIter,iterList)
    xcs.setPopulationSize(pop)
    xcs.setSubsumption(sub)
    xcs.setSelection(select)     

    #Run the XCS Algorithm  
    xcs.runXCS()
     
if __name__ == '__main__':
    main()
